<?php

require_once 'include/api/SugarApi.php';
require_once 'include/api/SugarApiException.php';
require_once 'custom/clients/base/api/SugarApiExceptionDuplicateKey.php';
require_once 'include/SugarQuery/SugarQuery.php';
require_once 'custom/PxRoutines/NewsFeedLogic/Collector/MobileReSync.php';

class PxGCMApi extends SugarApi {
    public function registerApiRest()
    {
        return array(
            'gcmRegistrationPost' => array(
                'reqType' => 'POST',
                'path' => array('gcm', 'registration', 'token'),
                'pathVars' => array('', '', ''),
                'method' => 'registerDevice',
                'exceptions' => array(
                    'SugarApiExceptionDuplicateKey',
                    'SugarApiExceptionInvalidParameter',
                ),
            ),
            'gcmMessagesGet' => array(
                'reqType' => 'GET',
                'path' => array('gcm', 'messages'),
                'pathVars' => array('', ''),
                'method' => 'getMessages',
            ),
        );
    }

    public function registerDevice(ServiceBase $api, array $args){
        $token = $this->getTokenFromHttpArguments($args);

		$mobileReSync = new MobileReSync();

		$gcmRegistrationId = $this->registrationTokenExists($token);

        if(!empty($gcmRegistrationId)) {
			$mobileReSync->refreshContactsByRegistration($gcmRegistrationId);
            throw new SugarApiExceptionDuplicateKey("EXCEPTION_DUPLICATE_GCM_REGISTRATION_TOKEN");
        }
        $gcmRegistration = $this->createRegistrationRecord($token);

		$mobileReSync->refreshContactsByRegistration($gcmRegistration->id);

        return $this->formatRegisterDeviceResponse($gcmRegistration);
    }

    private function registrationTokenExists($token){
        $query = $this->getRegistrationTokenCheckQuery($token);
        $row = $query->getOne();
        return !empty($row) ? $row : '';
    }

    private function getRegistrationTokenCheckQuery($token)
    {
        $query = new SugarQuery();
		$query->from(BeanFactory::getBean('PX_GCMRegistrations'));
		$query->select(array('id'));
        $query->where()->equals('registration_token', $token);
        return $query;
    }

    private function getTokenFromHttpArguments(array $args)
    {
        if (empty($args['registration_token'])) {
            throw new SugarApiExceptionInvalidParameter("EXCEPTION_INVALID_GCM_REGISTRATION_TOKEN");
        }
        $token = $args['registration_token'];
        return $token;
    }

    private function createRegistrationRecord($token){
        global $current_user;
        $gcmRegistration = BeanFactory::getBean('PX_GCMRegistrations');
        $gcmRegistration->assigned_user_id = $current_user->id;
        $gcmRegistration->registration_token = $token;
        $gcmRegistration->save(false);
        return $gcmRegistration;
    }

    private function formatRegisterDeviceResponse($gcmRegistration){
        return array(
            'id' => $gcmRegistration->id,
            'registration_token' => $gcmRegistration->registration_token,
        );
    }

    //<px_change author="Yazvetskyi" date="2016-05-24">
    private function getTimestampFromString($value){
        //$date = DateTime::createFromFormat('Y-M-d H:i:s', $value);
        $date = new DateTime($value);
        return $date->getTimestamp();
    }

    private function getStringFromTimesamp($value){
		$date = new DateTime();
		$date->setTimestamp($value);
		$timeZone = new DateTimeZone('UTC');
		$date->setTimezone($timeZone);

		return $date->format('Y-m-d H:i:s');
    }
    //</px_change>

    ///////////////////////////////////////////////////////////////////////////////////////////

    public function getMessages(ServiceBase $api, array $args){
        $query = $this->getMessagesQuery($args);

        $rows = $query->execute('array');

        $messages = $this->formatMessageRows($rows);
        return $messages;
    }

    private function getMessagesQuery(array $args){
        global $current_user;
        $query = new SugarQuery();
        $query->select(array(
            'px_gcmmessages.id',
            'px_gcmmessages.message_time',
            'px_gcmmessages.message_type',
            'px_gcmmessages.message',
        ));
        $query->from(BeanFactory::getBean('PX_GCMMessages'));
        $query->where()->equals('px_gcmmessages.assigned_user_id', $current_user->id);


        if(isset($args['registration_token'])) {
            $gcmRegistrationId = $this->getRegistrationIdFromHttpArguments($args);
            if(empty($gcmRegistrationId)){
                return new EmptyResultQuery();
            }
            $query->where()->equals('px_gcmmessages.gcmregistration_id', $gcmRegistrationId);
        }
        if(isset($args['message_type'])) {
            $query->where()->equals('px_gcmmessages.message_type', $args['message_type']);
        }
        if(isset($args['since'])){
            //<px_change author="Yazvetskyi" date="2016-05-24">
            //$query->where()->gte('px_gcmmessages.date_modified', $args['since']);
            $dateFrom = $this->getStringFromTimesamp($args['since']);
            $query->where()->gte('px_gcmmessages.message_time', $dateFrom);
            //</px_change>
        }
        $query->orderBy('px_gcmmessages.message_time', 'ASC');
        return $query;
    }

    private function getRegistrationIdFromHttpArguments(array $args){

		if(empty($args['registration_token'])){
            return "";
        }
        $registrationToken = $args['registration_token'];
        $query = new SugarQuery();
        $query->select(array(
            'id',
        ));
        $query->from(BeanFactory::getBean('PX_GCMRegistrations'));
        $query->where()->equals('registration_token', $registrationToken);
        $result = $query->getOne();



        return $result;
    }

    private function formatMessageRows(array $rows)
    {
        $result = array();
        foreach($rows as $row){
            //if(!empty($row['message'])){
			$result[] = $this->formatMessageRow($row);
           // }
        }
        return $result;
    }

	static $messageTypeMap = array(
		'mobile_sync' => 'update_contact',
		'delete_contact' => 'delete_contact',
	);

    private function formatMessageRow($row)
    {
		$messageRow = array(
			'timestamp' => $this->getTimestampFromString($row['date_modified']),
			'category' => 'sync',
			'type' => !empty(self::$messageTypeMap[$row['message_type']]) ? self::$messageTypeMap[$row['message_type']] : $row['message_type'],
		);
		if(!empty($row['message'])){
			$messageRow['content'] = json_decode($row['message']);
		}



        return $messageRow;
    }
}

class EmptyResultQuery extends SugarQuery {
    public function getOne()
    {
        return false;
    }

    public function execute($type = "array", $encode = true)
    {
        switch ($type) {
            case 'raw':
                return "";
            case 'db':
                return false;
            case 'json':
                return '[]';
            case 'array':
            default:
                return array();
        }
    }
}

